import os

from fsgs.drivers.gamedriver import GameDriver
from fsgs.option import Option
from fsgs.spectrum import ZXS_48_ROM, ZXS_128_0_ROM, ZXS_128_1_ROM, \
    ZXS_PLUS3_0_ROM, ZXS_PLUS3_1_ROM, ZXS_PLUS3_2_ROM, ZXS_PLUS3_3_ROM

ZXS_MODEL_48K = "spectrum"
ZXS_MODEL_128 = "spectrum128"
ZXS_MODEL_PLUS3 = "spectrum+3"


class FuseZXSDriver(GameDriver):
    JOYSTICK = {
        "type": "joystick",
        "description": "Joystick",
        "mapping_name": "zx-spectrum-joystick",
    }

    PORTS = [
        {
            "description": "Joystick Port",
            "types": [JOYSTICK]
        },
    ]

    def __init__(self, fsgs):
        super().__init__(fsgs)
        self.emulator.name = "fuse-fs"
        self.helper = SpectrumHelper(self.config)
        # self.emulator.allow_system_emulator = True
        self.media_dir = self.temp_dir("Media")
        self.roms_dir = self.temp_dir("ROMs")
        self.tos_file = self.temp_file("tos.img")
        # self.config_file = self.temp_file("hatari.cfg")
        # self.floppies = []

    def prepare(self):
        super().prepare()
        # self.prepare_tos()
        # self.prepare_floppies()
        # with open(self.config_file.path, "w", encoding="UTF-8") as f:
        #     self.write_config_file(f)
        # self.emulator.args.extend(["--configfile", self.config_file.path])

        def install_rom(name, rom):
            path = os.path.join(self.roms_dir.path, name + ".rom")
            self.files.add(path, sha1=rom.sha1, description=name + ".rom")
            self.emulator.args.extend(["--rom-{}".format(name), path])

        # if self.helper.model() == ZXS_MODEL_48K:
        #     install_rom("48", ZXS_48_ROM)
        # elif self.helper.model() == ZXS_MODEL_128:
        #     install_rom("128-0", ZXS_128_0_ROM)
        #     install_rom("128-1", ZXS_128_1_ROM)
        # elif self.helper.model() == ZXS_MODEL_PLUS3:
        #     install_rom("plus3-0", ZXS_PLUS3_0_ROM)
        #     install_rom("plus3-1", ZXS_PLUS3_1_ROM)
        #     install_rom("plus3-2", ZXS_PLUS3_2_ROM)
        #     install_rom("plus3-3", ZXS_PLUS3_3_ROM)

        if self.config.get(Option.TAPE_DRIVE_0):
            source = self.config.get(Option.TAPE_DRIVE_0)
            name = os.path.basename(source)
            path = os.path.join(self.media_dir.path, name)
            self.files.add(path, source=source)
            self.emulator.args.extend(["--tape", path])

        if self.config.get(Option.FLOPPY_DRIVE_0):
            source = self.config.get(Option.FLOPPY_DRIVE_0)
            name = os.path.basename(source)
            path = os.path.join(self.media_dir.path, name)
            self.files.add(path, source=source)
            self.emulator.args.extend(["--plus3disk", path])
            self.emulator.args.extend(["--drive-plus3b-type", "Disabled"])

        if self.use_auto_load():
            self.emulator.args.extend(["--auto-load"])
        if False:
            self.emulator.args.extend(["--fastload"])
        else:
            self.emulator.args.extend(["--no-fastload"])

        if self.helper.accuracy() == 1:
            traps = False
            # fastload = False
            accelerate_loader = False
        else:
            traps = True
            # fastload = True
            accelerate_loader = True
        self.emulator.args.extend(["--traps"] if traps else ["--no-traps"])
        if accelerate_loader:
            self.emulator.args.extend(["--accelerate-loader"])
        else:
            self.emulator.args.extend(["--no-accelerate-loader"])

        if self.use_fullscreen():
            self.emulator.args.extend(["--full-screen"])
        self.emulator.args.extend(["--graphics-filter", "2x"])

    def install(self):
        super().install()

    def run(self):
        super().run()

    def finish(self):
        super().finish()

    def get_game_refresh_rate(self):
        return self.helper.refresh_rate()


class SpectrumHelper:
    def __init__(self, config):
        self.config = config

    def accuracy(self):
        try:
            accuracy = int(self.config.get(Option.ACCURACY, "1"))
        except ValueError:
            accuracy = 1
        return accuracy

    def model(self):
        if self.config[Option.ZXS_MODEL] == "spectrum128":
            return ZXS_MODEL_128
        if self.config[Option.ZXS_MODEL] == "spectrum+3":
            return ZXS_MODEL_PLUS3
        return ZXS_MODEL_48K

    def refresh_rate(self):
        # Refresh rate values retrieved from MESS.
        if self.config[Option.ZXS_MODEL] == "spectrum128":
            return 50.021084
        else:
            return 50.080128
